'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _regeneratorRuntime = _interopDefault(require('@babel/runtime/regenerator'));

function ensureFunction(input) {
  return typeof input === 'function' ? input : function () {
    return input;
  };
}

function compose(fn) {
  for (var _len = arguments.length, fns = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    fns[_key - 1] = arguments[_key];
  }

  return fns.reduce(function (f, g) {
    var g_ = ensureFunction(g);
    return function () {
      return f(g_.apply(void 0, arguments));
    };
  }, ensureFunction(fn));
}

function pipe() {
  for (var _len = arguments.length, fns = new Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  return fns.reduceRight(function (f, g) {
    var g_ = ensureFunction(g);
    return function () {
      return f(g_.apply(void 0, arguments));
    };
  }, ensureFunction(fns.pop()));
}

function _asyncIterator(iterable) {
  var method;

  if (typeof Symbol !== "undefined") {
    if (Symbol.asyncIterator) {
      method = iterable[Symbol.asyncIterator];
      if (method != null) return method.call(iterable);
    }

    if (Symbol.iterator) {
      method = iterable[Symbol.iterator];
      if (method != null) return method.call(iterable);
    }
  }

  throw new TypeError("Object is not async iterable");
}

function _AwaitValue(value) {
  this.wrapped = value;
}

function _AsyncGenerator(gen) {
  var front, back;

  function send(key, arg) {
    return new Promise(function (resolve, reject) {
      var request = {
        key: key,
        arg: arg,
        resolve: resolve,
        reject: reject,
        next: null
      };

      if (back) {
        back = back.next = request;
      } else {
        front = back = request;
        resume(key, arg);
      }
    });
  }

  function resume(key, arg) {
    try {
      var result = gen[key](arg);
      var value = result.value;
      var wrappedAwait = value instanceof _AwaitValue;
      Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) {
        if (wrappedAwait) {
          resume(key === "return" ? "return" : "next", arg);
          return;
        }

        settle(result.done ? "return" : "normal", arg);
      }, function (err) {
        resume("throw", err);
      });
    } catch (err) {
      settle("throw", err);
    }
  }

  function settle(type, value) {
    switch (type) {
      case "return":
        front.resolve({
          value: value,
          done: true
        });
        break;

      case "throw":
        front.reject(value);
        break;

      default:
        front.resolve({
          value: value,
          done: false
        });
        break;
    }

    front = front.next;

    if (front) {
      resume(front.key, front.arg);
    } else {
      back = null;
    }
  }

  this._invoke = send;

  if (typeof gen.return !== "function") {
    this.return = undefined;
  }
}

if (typeof Symbol === "function" && Symbol.asyncIterator) {
  _AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
    return this;
  };
}

_AsyncGenerator.prototype.next = function (arg) {
  return this._invoke("next", arg);
};

_AsyncGenerator.prototype.throw = function (arg) {
  return this._invoke("throw", arg);
};

_AsyncGenerator.prototype.return = function (arg) {
  return this._invoke("return", arg);
};

function _wrapAsyncGenerator(fn) {
  return function () {
    return new _AsyncGenerator(fn.apply(this, arguments));
  };
}

function _awaitAsyncGenerator(value) {
  return new _AwaitValue(value);
}

function _asyncGeneratorDelegate(inner, awaitWrap) {
  var iter = {},
      waiting = false;

  function pump(key, value) {
    waiting = true;
    value = new Promise(function (resolve) {
      resolve(inner[key](value));
    });
    return {
      done: false,
      value: awaitWrap(value)
    };
  }

  if (typeof Symbol === "function" && Symbol.iterator) {
    iter[Symbol.iterator] = function () {
      return this;
    };
  }

  iter.next = function (value) {
    if (waiting) {
      waiting = false;
      return value;
    }

    return pump("next", value);
  };

  if (typeof inner.throw === "function") {
    iter.throw = function (value) {
      if (waiting) {
        waiting = false;
        throw value;
      }

      return pump("throw", value);
    };
  }

  if (typeof inner.return === "function") {
    iter.return = function (value) {
      if (waiting) {
        waiting = false;
        return value;
      }

      return pump("return", value);
    };
  }

  return iter;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(n);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o) {
  var i = 0;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  i = o[Symbol.iterator]();
  return i.next.bind(i);
}

function isIterable(input) {
  if (typeof input !== 'object' || input === null) {
    return false;
  }

  return input[Symbol.iterator] !== undefined;
}
function isAsyncIterable(input) {
  if (typeof input !== 'object' || input === null) {
    return false;
  }

  return input[Symbol.asyncIterator] !== undefined;
}

function concat() {
  var _ref2;

  for (var _len = arguments.length, data = new Array(_len), _key = 0; _key < _len; _key++) {
    data[_key] = arguments[_key];
  }

  if (data.some(isAsyncIterable) || data.some(function (datum) {
    return datum instanceof Promise;
  })) {
    var _ref;

    return _ref = {}, _ref[Symbol.asyncIterator] = function () {
      return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
        var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _context.prev = 2;
                _context.t0 = _asyncIterator;
                _context.next = 6;
                return _awaitAsyncGenerator(Promise.all(data));

              case 6:
                _context.t1 = _context.sent;
                _iterator = (0, _context.t0)(_context.t1);

              case 8:
                _context.next = 10;
                return _awaitAsyncGenerator(_iterator.next());

              case 10:
                _step = _context.sent;
                _iteratorNormalCompletion = _step.done;
                _context.next = 14;
                return _awaitAsyncGenerator(_step.value);

              case 14:
                _value = _context.sent;

                if (_iteratorNormalCompletion) {
                  _context.next = 21;
                  break;
                }

                datum = _value;
                return _context.delegateYield(_asyncGeneratorDelegate(_asyncIterator(datum), _awaitAsyncGenerator), "t2", 18);

              case 18:
                _iteratorNormalCompletion = true;
                _context.next = 8;
                break;

              case 21:
                _context.next = 27;
                break;

              case 23:
                _context.prev = 23;
                _context.t3 = _context["catch"](2);
                _didIteratorError = true;
                _iteratorError = _context.t3;

              case 27:
                _context.prev = 27;
                _context.prev = 28;

                if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                  _context.next = 32;
                  break;
                }

                _context.next = 32;
                return _awaitAsyncGenerator(_iterator["return"]());

              case 32:
                _context.prev = 32;

                if (!_didIteratorError) {
                  _context.next = 35;
                  break;
                }

                throw _iteratorError;

              case 35:
                return _context.finish(32);

              case 36:
                return _context.finish(27);

              case 37:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, null, [[2, 23, 27, 37], [28,, 32, 36]]);
      }))();
    }, _ref;
  }

  return _ref2 = {}, _ref2[Symbol.iterator] = function () {
    return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
      var _iterator2, _step2, datum;

      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _iterator2 = _createForOfIteratorHelperLoose(data);

            case 1:
              if ((_step2 = _iterator2()).done) {
                _context2.next = 6;
                break;
              }

              datum = _step2.value;
              return _context2.delegateYield(datum, "t0", 4);

            case 4:
              _context2.next = 1;
              break;

            case 6:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    })();
  }, _ref2;
}

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously implement a generic for loop
function _for(test, update, body) {
	var stage;
	for (;;) {
		var shouldContinue = test();
		if (_isSettledPact(shouldContinue)) {
			shouldContinue = shouldContinue.v;
		}
		if (!shouldContinue) {
			return result;
		}
		if (shouldContinue.then) {
			stage = 0;
			break;
		}
		var result = body();
		if (result && result.then) {
			if (_isSettledPact(result)) {
				result = result.s;
			} else {
				stage = 1;
				break;
			}
		}
		if (update) {
			var updateValue = update();
			if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
				stage = 2;
				break;
			}
		}
	}
	var pact = new _Pact();
	var reject = _settle.bind(null, pact, 2);
	(stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
	return pact;
	function _resumeAfterBody(value) {
		result = value;
		do {
			if (update) {
				updateValue = update();
				if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
					updateValue.then(_resumeAfterUpdate).then(void 0, reject);
					return;
				}
			}
			shouldContinue = test();
			if (!shouldContinue || (_isSettledPact(shouldContinue) && !shouldContinue.v)) {
				_settle(pact, 1, result);
				return;
			}
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
				return;
			}
			result = body();
			if (_isSettledPact(result)) {
				result = result.v;
			}
		} while (!result || !result.then);
		result.then(_resumeAfterBody).then(void 0, reject);
	}
	function _resumeAfterTest(shouldContinue) {
		if (shouldContinue) {
			result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
			} else {
				_resumeAfterBody(result);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
	function _resumeAfterUpdate() {
		if (shouldContinue = test()) {
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
			} else {
				_resumeAfterTest(shouldContinue);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
}

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

// Asynchronously await a promise and pass the result to a finally continuation
function _finallyRethrows(body, finalizer) {
	try {
		var result = body();
	} catch (e) {
		return finalizer(true, e);
	}
	if (result && result.then) {
		return result.then(finalizer.bind(null, false), finalizer.bind(null, true));
	}
	return finalizer(false, result);
}

function every(predicate) {
  return function everyFn(data) {
    if (data == null) {
      return false;
    }

    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp8 = function _temp8(stream) {
            var _exit = false;
            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp6 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                return _for(function () {
                  function _temp2(_iterator$next) {
                    function _temp(_step$value) {
                      return !_exit && (_value = _step$value, !_iteratorNormalCompletion);
                    }

                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return _exit ? _temp(!_exit && _step.value) : Promise.resolve(!_exit && _step.value).then(_temp);
                  }

                  return _exit ? !!_temp2(!_exit && _iterator.next()) : !!Promise.resolve(!_exit && _iterator.next()).then(_temp2);
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;

                  if (!predicate(datum, i++)) {
                    _exit = true;
                    return false;
                  }
                });
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result2) {
              function _temp5(_result3) {
                if (_exit) return _result3;
                if (_wasThrown) throw _result2;
                return _result2;
              }

              var _temp4 = _finallyRethrows(function () {
                var _temp3 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp3 && _temp3.then) return _temp3.then(function () {});
              }, function (_wasThrown2, _result3) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result3;
                return _result3;
              });

              return _temp4 && _temp4.then ? _temp4.then(_temp5) : _temp5(_temp4);
            });

            return _temp6 && _temp6.then ? _temp6.then(function (_result2) {
              return _exit ? _result2 : true;
            }) : _exit ? _temp6 : true;
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp8) : _temp8(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;

      if (!predicate(datum, i++)) {
        return false;
      }
    }

    return true;
  };
}

function filter(fn) {
  return function filterFn(data) {
    var _ref2;

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, i, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  i = 0;
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 11;
                  _iterator = _asyncIterator(stream);

                case 13:
                  _context.next = 15;
                  return _awaitAsyncGenerator(_iterator.next());

                case 15:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 19;
                  return _awaitAsyncGenerator(_step.value);

                case 19:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 29;
                    break;
                  }

                  datum = _value;

                  if (fn(datum, i++)) {
                    _context.next = 24;
                    break;
                  }

                  return _context.abrupt("continue", 26);

                case 24:
                  _context.next = 26;
                  return datum;

                case 26:
                  _iteratorNormalCompletion = true;
                  _context.next = 13;
                  break;

                case 29:
                  _context.next = 35;
                  break;

                case 31:
                  _context.prev = 31;
                  _context.t1 = _context["catch"](11);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 35:
                  _context.prev = 35;
                  _context.prev = 36;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 40;
                    break;
                  }

                  _context.next = 40;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 40:
                  _context.prev = 40;

                  if (!_didIteratorError) {
                    _context.next = 43;
                    break;
                  }

                  throw _iteratorError;

                case 43:
                  return _context.finish(40);

                case 44:
                  return _context.finish(35);

                case 45:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[11, 31, 35, 45], [36,, 40, 44]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var i, _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                i = 0;
                _iterator2 = _createForOfIteratorHelperLoose(data);

              case 2:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 10;
                  break;
                }

                datum = _step2.value;

                if (fn(datum, i++)) {
                  _context2.next = 6;
                  break;
                }

                return _context2.abrupt("continue", 8);

              case 6:
                _context2.next = 8;
                return datum;

              case 8:
                _context2.next = 2;
                break;

              case 10:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function find(predicate) {
  return function findFn(data) {
    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp8 = function _temp8(stream) {
            var _exit = false;
            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp6 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                return _for(function () {
                  function _temp2(_iterator$next) {
                    function _temp(_step$value) {
                      return !_exit && (_value = _step$value, !_iteratorNormalCompletion);
                    }

                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return _exit ? _temp(!_exit && _step.value) : Promise.resolve(!_exit && _step.value).then(_temp);
                  }

                  return _exit ? !!_temp2(!_exit && _iterator.next()) : !!Promise.resolve(!_exit && _iterator.next()).then(_temp2);
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;

                  if (predicate(datum, i++)) {
                    _exit = true;
                    return datum;
                  }
                });
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result2) {
              function _temp5(_result3) {
                if (_exit) return _result3;
                if (_wasThrown) throw _result2;
                return _result2;
              }

              var _temp4 = _finallyRethrows(function () {
                var _temp3 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp3 && _temp3.then) return _temp3.then(function () {});
              }, function (_wasThrown2, _result3) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result3;
                return _result3;
              });

              return _temp4 && _temp4.then ? _temp4.then(_temp5) : _temp5(_temp4);
            });

            return _temp6 && _temp6.then ? _temp6.then(function (_result2) {
              return _exit ? _result2 : undefined;
            }) : _exit ? _temp6 : undefined;
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp8) : _temp8(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;

      if (predicate(datum, i++)) {
        return datum;
      }
    }

    return undefined;
  };
}

function findIndex(predicate) {
  return function findIndexFn(data) {
    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp9 = function _temp9(stream) {
            var _exit = false;

            function _temp7(_result2) {
              return _exit ? _result2 : -1;
            }

            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp6 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                return _for(function () {
                  function _temp2(_iterator$next) {
                    function _temp(_step$value) {
                      return !_exit && (_value = _step$value, !_iteratorNormalCompletion);
                    }

                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return _exit ? _temp(!_exit && _step.value) : Promise.resolve(!_exit && _step.value).then(_temp);
                  }

                  return _exit ? !!_temp2(!_exit && _iterator.next()) : !!Promise.resolve(!_exit && _iterator.next()).then(_temp2);
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;

                  if (predicate(datum, i)) {
                    _exit = true;
                    return i;
                  }

                  i++;
                });
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result2) {
              function _temp5(_result3) {
                if (_exit) return _result3;
                if (_wasThrown) throw _result2;
                return _result2;
              }

              var _temp4 = _finallyRethrows(function () {
                var _temp3 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp3 && _temp3.then) return _temp3.then(function () {});
              }, function (_wasThrown2, _result3) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result3;
                return _result3;
              });

              return _temp4 && _temp4.then ? _temp4.then(_temp5) : _temp5(_temp4);
            });

            return _temp6 && _temp6.then ? _temp6.then(_temp7) : _temp7(_temp6);
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp9) : _temp9(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;

      if (predicate(datum, i)) {
        return i;
      }

      i++;
    }

    return -1;
  };
}

function map(fn) {
  return function mapFn(data) {
    var _ref2;

    if (data == null) {
      return;
    } // Handle the async version


    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, i, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  i = 0;
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 11;
                  _iterator = _asyncIterator(stream);

                case 13:
                  _context.next = 15;
                  return _awaitAsyncGenerator(_iterator.next());

                case 15:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 19;
                  return _awaitAsyncGenerator(_step.value);

                case 19:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 27;
                    break;
                  }

                  datum = _value;
                  _context.next = 24;
                  return fn(datum, i++);

                case 24:
                  _iteratorNormalCompletion = true;
                  _context.next = 13;
                  break;

                case 27:
                  _context.next = 33;
                  break;

                case 29:
                  _context.prev = 29;
                  _context.t1 = _context["catch"](11);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 33:
                  _context.prev = 33;
                  _context.prev = 34;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 38;
                    break;
                  }

                  _context.next = 38;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 38:
                  _context.prev = 38;

                  if (!_didIteratorError) {
                    _context.next = 41;
                    break;
                  }

                  throw _iteratorError;

                case 41:
                  return _context.finish(38);

                case 42:
                  return _context.finish(33);

                case 43:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[11, 29, 33, 43], [34,, 38, 42]]);
        }))();
      }, _ref;
    } // Handle the sync version


    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var i, _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                i = 0;
                _iterator2 = _createForOfIteratorHelperLoose(data);

              case 2:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 8;
                  break;
                }

                datum = _step2.value;
                _context2.next = 6;
                return fn(datum, i++);

              case 6:
                _context2.next = 2;
                break;

              case 8:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function reduce(fn, initial) {
  return function reduceFn(data) {
    if (data == null) {
      return;
    }

    var acc = initial;

    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp7 = function _temp7(stream) {
            var _exit = false;
            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp5 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                var _temp = _for(function () {
                  return !!Promise.resolve(_iterator.next()).then(function (_iterator$next) {
                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return Promise.resolve(_step.value).then(function (_step$value) {
                      return _value = _step$value, !_iteratorNormalCompletion;
                    });
                  });
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;
                  acc = fn(acc, datum, i++);
                });

                if (_temp && _temp.then) return _temp.then(function () {});
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result) {
              function _temp4(_result2) {
                if (_exit) return _result2;
                if (_wasThrown) throw _result;
                return _result;
              }

              var _temp3 = _finallyRethrows(function () {
                var _temp2 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp2 && _temp2.then) return _temp2.then(function () {});
              }, function (_wasThrown2, _result2) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result2;
                return _result2;
              });

              return _temp3 && _temp3.then ? _temp3.then(_temp4) : _temp4(_temp3);
            });

            return _temp5 && _temp5.then ? _temp5.then(function (_result) {
              return _exit ? _result : acc;
            }) : _exit ? _temp5 : acc;
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp7) : _temp7(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;
      acc = fn(acc, datum, i++);
    }

    return acc;
  };
}

function toArray() {
  return function (data) {
    return reduce(function (acc, current) {
      acc.push(current);
      return acc;
    }, [])(data);
  };
}

/**
 * This is pretty slow because it has to first go through the whole iterator
 * (to make it an array), then reverse the whole thing and then start
 * yielding again.
 */

function reverse() {
  return function reverseFn(data) {
    var _ref2;

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, program, array, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  program = pipe(toArray());
                  _context.next = 11;
                  return _awaitAsyncGenerator(program(stream));

                case 11:
                  array = _context.sent;
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 14;
                  _iterator = _asyncIterator(array.reverse());

                case 16:
                  _context.next = 18;
                  return _awaitAsyncGenerator(_iterator.next());

                case 18:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 22;
                  return _awaitAsyncGenerator(_step.value);

                case 22:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 30;
                    break;
                  }

                  datum = _value;
                  _context.next = 27;
                  return datum;

                case 27:
                  _iteratorNormalCompletion = true;
                  _context.next = 16;
                  break;

                case 30:
                  _context.next = 36;
                  break;

                case 32:
                  _context.prev = 32;
                  _context.t1 = _context["catch"](14);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 36:
                  _context.prev = 36;
                  _context.prev = 37;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 41;
                    break;
                  }

                  _context.next = 41;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 41:
                  _context.prev = 41;

                  if (!_didIteratorError) {
                    _context.next = 44;
                    break;
                  }

                  throw _iteratorError;

                case 44:
                  return _context.finish(41);

                case 45:
                  return _context.finish(36);

                case 46:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[14, 32, 36, 46], [37,, 41, 45]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _iterator2 = _createForOfIteratorHelperLoose(Array.from(data).reverse());

              case 1:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 7;
                  break;
                }

                datum = _step2.value;
                _context2.next = 5;
                return datum;

              case 5:
                _context2.next = 1;
                break;

              case 7:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function slice(begin, end) {
  if (begin === void 0) {
    begin = 0;
  }

  if (end === void 0) {
    end = Infinity;
  }

  return function sliceFn(data) {
    var _ref2;

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, iterator, local_begin, local_end, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  iterator = stream;
                  local_begin = begin;
                  local_end = end - local_begin; // Skip the first X values

                  while (local_begin-- > 0) {
                    iterator.next();
                  } // Loop through the remaining items until the end is reached


                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 14;
                  _iterator = _asyncIterator(iterator);

                case 16:
                  _context.next = 18;
                  return _awaitAsyncGenerator(_iterator.next());

                case 18:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 22;
                  return _awaitAsyncGenerator(_step.value);

                case 22:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 32;
                    break;
                  }

                  datum = _value;
                  _context.next = 27;
                  return datum;

                case 27:
                  if (!(--local_end < 0)) {
                    _context.next = 29;
                    break;
                  }

                  return _context.abrupt("return");

                case 29:
                  _iteratorNormalCompletion = true;
                  _context.next = 16;
                  break;

                case 32:
                  _context.next = 38;
                  break;

                case 34:
                  _context.prev = 34;
                  _context.t1 = _context["catch"](14);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 38:
                  _context.prev = 38;
                  _context.prev = 39;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 43;
                    break;
                  }

                  _context.next = 43;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 43:
                  _context.prev = 43;

                  if (!_didIteratorError) {
                    _context.next = 46;
                    break;
                  }

                  throw _iteratorError;

                case 46:
                  return _context.finish(43);

                case 47:
                  return _context.finish(38);

                case 48:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[14, 34, 38, 48], [39,, 43, 47]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var iterator, local_begin, local_end, _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                iterator = Array.isArray(data) ? data[Symbol.iterator]() : data;
                local_begin = begin;
                local_end = end - local_begin; // Skip the first X values

                while (local_begin-- > 0) {
                  iterator.next();
                } // Loop through the remaining items until the end is reached


                _iterator2 = _createForOfIteratorHelperLoose(iterator);

              case 5:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 13;
                  break;
                }

                datum = _step2.value;
                _context2.next = 9;
                return datum;

              case 9:
                if (!(--local_end < 0)) {
                  _context2.next = 11;
                  break;
                }

                return _context2.abrupt("return");

              case 11:
                _context2.next = 5;
                break;

              case 13:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function some(predicate) {
  return function someFn(data) {
    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp8 = function _temp8(stream) {
            var _exit = false;
            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp6 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                return _for(function () {
                  function _temp2(_iterator$next) {
                    function _temp(_step$value) {
                      return !_exit && (_value = _step$value, !_iteratorNormalCompletion);
                    }

                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return _exit ? _temp(!_exit && _step.value) : Promise.resolve(!_exit && _step.value).then(_temp);
                  }

                  return _exit ? !!_temp2(!_exit && _iterator.next()) : !!Promise.resolve(!_exit && _iterator.next()).then(_temp2);
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;

                  if (predicate(datum, i++)) {
                    _exit = true;
                    return true;
                  }
                });
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result2) {
              function _temp5(_result3) {
                if (_exit) return _result3;
                if (_wasThrown) throw _result2;
                return _result2;
              }

              var _temp4 = _finallyRethrows(function () {
                var _temp3 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp3 && _temp3.then) return _temp3.then(function () {});
              }, function (_wasThrown2, _result3) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result3;
                return _result3;
              });

              return _temp4 && _temp4.then ? _temp4.then(_temp5) : _temp5(_temp4);
            });

            return _temp6 && _temp6.then ? _temp6.then(function (_result2) {
              return _exit ? _result2 : false;
            }) : _exit ? _temp6 : false;
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp8) : _temp8(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;

      if (predicate(datum, i++)) {
        return true;
      }
    }

    return false;
  };
}

function chunk(size) {
  return function chunkFn(data) {
    var _ref2;

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, chunk, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  // Let's have a placeholder for our current chunk
                  chunk = []; // Loop over our data

                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 11;
                  _iterator = _asyncIterator(stream);

                case 13:
                  _context.next = 15;
                  return _awaitAsyncGenerator(_iterator.next());

                case 15:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 19;
                  return _awaitAsyncGenerator(_step.value);

                case 19:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 30;
                    break;
                  }

                  datum = _value;
                  // Add item to our current chunk
                  chunk.push(datum);

                  if (!(chunk.length === size)) {
                    _context.next = 27;
                    break;
                  }

                  _context.next = 26;
                  return chunk;

                case 26:
                  // Let's also clear our chunk for the next chunk
                  chunk = [];

                case 27:
                  _iteratorNormalCompletion = true;
                  _context.next = 13;
                  break;

                case 30:
                  _context.next = 36;
                  break;

                case 32:
                  _context.prev = 32;
                  _context.t1 = _context["catch"](11);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 36:
                  _context.prev = 36;
                  _context.prev = 37;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 41;
                    break;
                  }

                  _context.next = 41;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 41:
                  _context.prev = 41;

                  if (!_didIteratorError) {
                    _context.next = 44;
                    break;
                  }

                  throw _iteratorError;

                case 44:
                  return _context.finish(41);

                case 45:
                  return _context.finish(36);

                case 46:
                  if (!(chunk.length > 0)) {
                    _context.next = 49;
                    break;
                  }

                  _context.next = 49;
                  return chunk;

                case 49:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[11, 32, 36, 46], [37,, 41, 45]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var chunk, _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                // Let's have a placeholder for our current chunk
                chunk = []; // Loop over our data

                _iterator2 = _createForOfIteratorHelperLoose(data);

              case 2:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 11;
                  break;
                }

                datum = _step2.value;
                // Add item to our current chunk
                chunk.push(datum);

                if (!(chunk.length === size)) {
                  _context2.next = 9;
                  break;
                }

                _context2.next = 8;
                return chunk;

              case 8:
                // Let's also clear our chunk for the next chunk
                chunk = [];

              case 9:
                _context2.next = 2;
                break;

              case 11:
                if (!(chunk.length > 0)) {
                  _context2.next = 14;
                  break;
                }

                _context2.next = 14;
                return chunk;

              case 14:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function head() {
  return function headFn(data) {
    if (data == null) {
      return;
    }

    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp8 = function _temp8(stream) {
            var _exit = false;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp6 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                return _for(function () {
                  function _temp2(_iterator$next) {
                    function _temp(_step$value) {
                      return !_exit && (_value = _step$value, !_iteratorNormalCompletion);
                    }

                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return _exit ? _temp(!_exit && _step.value) : Promise.resolve(!_exit && _step.value).then(_temp);
                  }

                  return _exit ? !!_temp2(!_exit && _iterator.next()) : !!Promise.resolve(!_exit && _iterator.next()).then(_temp2);
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;
                  _exit = true;
                  return datum;
                });
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result2) {
              function _temp5(_result3) {
                if (_exit) return _result3;
                if (_wasThrown) throw _result2;
                return _result2;
              }

              var _temp4 = _finallyRethrows(function () {
                var _temp3 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp3 && _temp3.then) return _temp3.then(function () {});
              }, function (_wasThrown2, _result3) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result3;
                return _result3;
              });

              return _temp4 && _temp4.then ? _temp4.then(_temp5) : _temp5(_temp4);
            });

            return _temp6 && _temp6.then ? _temp6.then(function (_result2) {
              return _exit ? _result2 : undefined;
            }) : _exit ? _temp6 : undefined;
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp8) : _temp8(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;
      return datum;
    }

    return undefined;
  };
} // Alias

var first = head;

function average() {
  return function averageFn(data) {
    var program = pipe(reduce(function (acc, current) {
      acc[0] += current;
      acc[1] += 1;
      return acc;
    }, [0, 0]), chunk(2), map(function (_ref) {
      var sum = _ref[0],
          count = _ref[1];
      return sum / count;
    }), head());
    return program(data);
  };
} // Alias

var mean = average;

function compact() {
  return filter(Boolean);
}

function sleep(ms) {
  return new Promise(function (resolve) {
    return setTimeout(resolve, ms);
  });
}

function delay(ms) {
  return /*#__PURE__*/function () {
    var _delayFn = _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(data) {
      var stream, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!(data == null)) {
                _context.next = 2;
                break;
              }

              return _context.abrupt("return");

            case 2:
              if (!(data instanceof Promise)) {
                _context.next = 8;
                break;
              }

              _context.next = 5;
              return _awaitAsyncGenerator(data);

            case 5:
              _context.t0 = _context.sent;
              _context.next = 9;
              break;

            case 8:
              _context.t0 = data;

            case 9:
              stream = _context.t0;
              _iteratorNormalCompletion = true;
              _didIteratorError = false;
              _context.prev = 12;
              _iterator = _asyncIterator(stream);

            case 14:
              _context.next = 16;
              return _awaitAsyncGenerator(_iterator.next());

            case 16:
              _step = _context.sent;
              _iteratorNormalCompletion = _step.done;
              _context.next = 20;
              return _awaitAsyncGenerator(_step.value);

            case 20:
              _value = _context.sent;

              if (_iteratorNormalCompletion) {
                _context.next = 30;
                break;
              }

              datum = _value;
              _context.next = 25;
              return _awaitAsyncGenerator(sleep(ms));

            case 25:
              _context.next = 27;
              return datum;

            case 27:
              _iteratorNormalCompletion = true;
              _context.next = 14;
              break;

            case 30:
              _context.next = 36;
              break;

            case 32:
              _context.prev = 32;
              _context.t1 = _context["catch"](12);
              _didIteratorError = true;
              _iteratorError = _context.t1;

            case 36:
              _context.prev = 36;
              _context.prev = 37;

              if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                _context.next = 41;
                break;
              }

              _context.next = 41;
              return _awaitAsyncGenerator(_iterator["return"]());

            case 41:
              _context.prev = 41;

              if (!_didIteratorError) {
                _context.next = 44;
                break;
              }

              throw _iteratorError;

            case 44:
              return _context.finish(41);

            case 45:
              return _context.finish(36);

            case 46:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[12, 32, 36, 46], [37,, 41, 45]]);
    }));

    function delayFn(_x) {
      return _delayFn.apply(this, arguments);
    }

    return delayFn;
  }();
}

function flatten(options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$shallow = _options.shallow,
      shallow = _options$shallow === void 0 ? false : _options$shallow;
  return function flattenFn(data) {
    var _ref2;

    if (data == null) {
      return;
    }

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 10;
                  _iterator = _asyncIterator(stream);

                case 12:
                  _context.next = 14;
                  return _awaitAsyncGenerator(_iterator.next());

                case 14:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 18;
                  return _awaitAsyncGenerator(_step.value);

                case 18:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 41;
                    break;
                  }

                  datum = _value;

                  if (!shallow) {
                    _context.next = 30;
                    break;
                  }

                  if (!isAsyncIterable(datum)) {
                    _context.next = 26;
                    break;
                  }

                  return _context.delegateYield(_asyncGeneratorDelegate(_asyncIterator(datum), _awaitAsyncGenerator), "t1", 24);

                case 24:
                  _context.next = 28;
                  break;

                case 26:
                  _context.next = 28;
                  return datum;

                case 28:
                  _context.next = 38;
                  break;

                case 30:
                  _context.t2 = _asyncGeneratorDelegate;
                  _context.t3 = _asyncIterator;
                  _context.next = 34;
                  return _awaitAsyncGenerator(flattenFn(datum));

                case 34:
                  _context.t4 = _context.sent;
                  _context.t5 = (0, _context.t3)(_context.t4);
                  _context.t6 = _awaitAsyncGenerator;
                  return _context.delegateYield((0, _context.t2)(_context.t5, _context.t6), "t7", 38);

                case 38:
                  _iteratorNormalCompletion = true;
                  _context.next = 12;
                  break;

                case 41:
                  _context.next = 47;
                  break;

                case 43:
                  _context.prev = 43;
                  _context.t8 = _context["catch"](10);
                  _didIteratorError = true;
                  _iteratorError = _context.t8;

                case 47:
                  _context.prev = 47;
                  _context.prev = 48;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 52;
                    break;
                  }

                  _context.next = 52;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 52:
                  _context.prev = 52;

                  if (!_didIteratorError) {
                    _context.next = 55;
                    break;
                  }

                  throw _iteratorError;

                case 55:
                  return _context.finish(52);

                case 56:
                  return _context.finish(47);

                case 57:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[10, 43, 47, 57], [48,, 52, 56]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (Array.isArray(data)) {
                  _context2.next = 5;
                  break;
                }

                _context2.next = 3;
                return data;

              case 3:
                _context2.next = 20;
                break;

              case 5:
                _iterator2 = _createForOfIteratorHelperLoose(data);

              case 6:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 20;
                  break;
                }

                datum = _step2.value;

                if (!shallow) {
                  _context2.next = 17;
                  break;
                }

                if (!isIterable(datum)) {
                  _context2.next = 13;
                  break;
                }

                return _context2.delegateYield(datum, "t0", 11);

              case 11:
                _context2.next = 15;
                break;

              case 13:
                _context2.next = 15;
                return datum;

              case 15:
                _context2.next = 18;
                break;

              case 17:
                return _context2.delegateYield(flattenFn(datum), "t1", 18);

              case 18:
                _context2.next = 6;
                break;

              case 20:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

var _marked = /*#__PURE__*/_regeneratorRuntime.mark(generate);

function generate(generator) {
  return _regeneratorRuntime.wrap(function generate$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:

          _context.next = 3;
          return generator();

        case 3:
          _context.next = 0;
          break;

        case 5:
        case "end":
          return _context.stop();
      }
    }
  }, _marked);
}

function groupBy(keySelector) {
  return function groupByFn(data) {
    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp7 = function _temp7(stream) {
            var _exit = false;
            var map = {};
            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp5 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                var _temp = _for(function () {
                  return !!Promise.resolve(_iterator.next()).then(function (_iterator$next) {
                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return Promise.resolve(_step.value).then(function (_step$value) {
                      return _value = _step$value, !_iteratorNormalCompletion;
                    });
                  });
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;
                  var key = keySelector(datum, i++);

                  if (map[key] === undefined) {
                    map[key] = [];
                  }

                  map[key].push(datum);
                });

                if (_temp && _temp.then) return _temp.then(function () {});
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result) {
              function _temp4(_result2) {
                if (_exit) return _result2;
                if (_wasThrown) throw _result;
                return _result;
              }

              var _temp3 = _finallyRethrows(function () {
                var _temp2 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp2 && _temp2.then) return _temp2.then(function () {});
              }, function (_wasThrown2, _result2) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result2;
                return _result2;
              });

              return _temp3 && _temp3.then ? _temp3.then(_temp4) : _temp4(_temp3);
            });

            return _temp5 && _temp5.then ? _temp5.then(function (_result) {
              return _exit ? _result : map;
            }) : _exit ? _temp5 : map;
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp7) : _temp7(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var map = {};
    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;
      var key = keySelector(datum, i++);

      if (map[key] === undefined) {
        map[key] = [];
      }

      map[key].push(datum);
    }

    return map;
  };
}

function max() {
  return reduce(function (lhs, rhs) {
    return Math.max(lhs, rhs);
  }, -Infinity);
}

function min() {
  return reduce(function (lhs, rhs) {
    return Math.min(lhs, rhs);
  }, Infinity);
}

function partition(predicate) {
  return function partitionFn(data) {
    if (data == null) {
      return;
    }

    if (isAsyncIterable(data) || data instanceof Promise) {
      return function () {
        try {
          var _temp8 = function _temp8(stream) {
            var _exit = false;

            function _temp6(_result) {
              return _exit ? _result : [a, b];
            }

            var a = [];
            var b = [];
            var i = 0;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;

            var _iteratorError;

            var _temp5 = _finallyRethrows(function () {
              return _catch(function () {
                var _iterator = _asyncIterator(stream),
                    _step,
                    _value;

                var _temp = _for(function () {
                  return !!Promise.resolve(_iterator.next()).then(function (_iterator$next) {
                    var _iteratorNormalComple = _iteratorNormalCompletion = _step.done,
                        _step2 = _step = _iterator$next;

                    return Promise.resolve(_step.value).then(function (_step$value) {
                      return _value = _step$value, !_iteratorNormalCompletion;
                    });
                  });
                }, function () {
                  return !!(_iteratorNormalCompletion = true);
                }, function () {
                  var datum = _value;

                  if (predicate(datum, i++)) {
                    a.push(datum);
                  } else {
                    b.push(datum);
                  }
                });

                if (_temp && _temp.then) return _temp.then(function () {});
              }, function (err) {
                _didIteratorError = true;
                _iteratorError = err;
              });
            }, function (_wasThrown, _result) {
              function _temp4(_result2) {
                if (_exit) return _result2;
                if (_wasThrown) throw _result;
                return _result;
              }

              var _temp3 = _finallyRethrows(function () {
                var _temp2 = function () {
                  if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                    return Promise.resolve(_iterator["return"]()).then(function () {});
                  }
                }();

                if (_temp2 && _temp2.then) return _temp2.then(function () {});
              }, function (_wasThrown2, _result2) {
                if (_didIteratorError) {
                  throw _iteratorError;
                }

                if (_wasThrown2) throw _result2;
                return _result2;
              });

              return _temp3 && _temp3.then ? _temp3.then(_temp4) : _temp4(_temp3);
            });

            return _temp5 && _temp5.then ? _temp5.then(_temp6) : _temp6(_temp5);
          };

          return Promise.resolve(data instanceof Promise ? Promise.resolve(data instanceof Promise ? data : data).then(_temp8) : _temp8(data instanceof Promise ? data : data));
        } catch (e) {
          return Promise.reject(e);
        }
      }();
    }

    var a = [];
    var b = [];
    var i = 0;

    for (var _iterator2 = _createForOfIteratorHelperLoose(data), _step3; !(_step3 = _iterator2()).done;) {
      var datum = _step3.value;

      if (predicate(datum, i++)) {
        a.push(datum);
      } else {
        b.push(datum);
      }
    }

    return [a, b];
  };
}

function clamp(input, min, max) {
  if (min === void 0) {
    min = Number.MIN_SAFE_INTEGER;
  }

  if (max === void 0) {
    max = Number.MAX_SAFE_INTEGER;
  }

  return Math.min(max, Math.max(min, input));
}

var _marked$1 = /*#__PURE__*/_regeneratorRuntime.mark(range);
function range(lowerbound, upperbound, step) {
  var fixed_step, lowerbound_, upperbound_, i;
  return _regeneratorRuntime.wrap(function range$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          if (step === void 0) {
            step = 1;
          }

          fixed_step = clamp(lowerbound < upperbound ? step > 0 ? step : -step : step > 0 ? -step : step);
          lowerbound_ = clamp(lowerbound);
          upperbound_ = clamp(upperbound);
          i = lowerbound_;

        case 5:
          if (!(lowerbound_ < upperbound_ ? i <= upperbound_ : i >= upperbound_)) {
            _context.next = 11;
            break;
          }

          _context.next = 8;
          return i;

        case 8:
          i += fixed_step;
          _context.next = 5;
          break;

        case 11:
        case "end":
          return _context.stop();
      }
    }
  }, _marked$1);
}

function skip(amount) {
  return slice(Math.max(0, amount));
}

function sum() {
  return reduce(function (total, current) {
    return total + current;
  }, 0);
}

function take(amount) {
  return slice(0, Math.max(0, amount - 1));
}

function takeWhile(fn) {
  return function takeWhileFn(data) {
    var _ref2;

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, i, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  i = 0;
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 11;
                  _iterator = _asyncIterator(stream);

                case 13:
                  _context.next = 15;
                  return _awaitAsyncGenerator(_iterator.next());

                case 15:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 19;
                  return _awaitAsyncGenerator(_step.value);

                case 19:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 29;
                    break;
                  }

                  datum = _value;

                  if (fn(datum, i++)) {
                    _context.next = 24;
                    break;
                  }

                  return _context.abrupt("return");

                case 24:
                  _context.next = 26;
                  return datum;

                case 26:
                  _iteratorNormalCompletion = true;
                  _context.next = 13;
                  break;

                case 29:
                  _context.next = 35;
                  break;

                case 31:
                  _context.prev = 31;
                  _context.t1 = _context["catch"](11);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 35:
                  _context.prev = 35;
                  _context.prev = 36;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 40;
                    break;
                  }

                  _context.next = 40;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 40:
                  _context.prev = 40;

                  if (!_didIteratorError) {
                    _context.next = 43;
                    break;
                  }

                  throw _iteratorError;

                case 43:
                  return _context.finish(40);

                case 44:
                  return _context.finish(35);

                case 45:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[11, 31, 35, 45], [36,, 40, 44]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var i, _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                i = 0;
                _iterator2 = _createForOfIteratorHelperLoose(data);

              case 2:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 10;
                  break;
                }

                datum = _step2.value;

                if (fn(datum, i++)) {
                  _context2.next = 6;
                  break;
                }

                return _context2.abrupt("return");

              case 6:
                _context2.next = 8;
                return datum;

              case 8:
                _context2.next = 2;
                break;

              case 10:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function tap(fn) {
  return map(function (datum, index) {
    fn(datum, index);
    return datum;
  });
}

function unique() {
  return function uniqueFn(data) {
    var _ref2;

    var seen = new Set([]);

    if (isAsyncIterable(data) || data instanceof Promise) {
      var _ref;

      return _ref = {}, _ref[Symbol.asyncIterator] = function () {
        return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
          var stream, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _value, datum;

          return _regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!(data instanceof Promise)) {
                    _context.next = 6;
                    break;
                  }

                  _context.next = 3;
                  return _awaitAsyncGenerator(data);

                case 3:
                  _context.t0 = _context.sent;
                  _context.next = 7;
                  break;

                case 6:
                  _context.t0 = data;

                case 7:
                  stream = _context.t0;
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _context.prev = 10;
                  _iterator = _asyncIterator(stream);

                case 12:
                  _context.next = 14;
                  return _awaitAsyncGenerator(_iterator.next());

                case 14:
                  _step = _context.sent;
                  _iteratorNormalCompletion = _step.done;
                  _context.next = 18;
                  return _awaitAsyncGenerator(_step.value);

                case 18:
                  _value = _context.sent;

                  if (_iteratorNormalCompletion) {
                    _context.next = 28;
                    break;
                  }

                  datum = _value;

                  if (seen.has(datum)) {
                    _context.next = 25;
                    break;
                  }

                  seen.add(datum);
                  _context.next = 25;
                  return datum;

                case 25:
                  _iteratorNormalCompletion = true;
                  _context.next = 12;
                  break;

                case 28:
                  _context.next = 34;
                  break;

                case 30:
                  _context.prev = 30;
                  _context.t1 = _context["catch"](10);
                  _didIteratorError = true;
                  _iteratorError = _context.t1;

                case 34:
                  _context.prev = 34;
                  _context.prev = 35;

                  if (!(!_iteratorNormalCompletion && _iterator["return"] != null)) {
                    _context.next = 39;
                    break;
                  }

                  _context.next = 39;
                  return _awaitAsyncGenerator(_iterator["return"]());

                case 39:
                  _context.prev = 39;

                  if (!_didIteratorError) {
                    _context.next = 42;
                    break;
                  }

                  throw _iteratorError;

                case 42:
                  return _context.finish(39);

                case 43:
                  return _context.finish(34);

                case 44:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[10, 30, 34, 44], [35,, 39, 43]]);
        }))();
      }, _ref;
    }

    return _ref2 = {}, _ref2[Symbol.iterator] = function () {
      return /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var _iterator2, _step2, datum;

        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _iterator2 = _createForOfIteratorHelperLoose(data);

              case 1:
                if ((_step2 = _iterator2()).done) {
                  _context2.next = 9;
                  break;
                }

                datum = _step2.value;

                if (seen.has(datum)) {
                  _context2.next = 7;
                  break;
                }

                seen.add(datum);
                _context2.next = 7;
                return datum;

              case 7:
                _context2.next = 1;
                break;

              case 9:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      })();
    }, _ref2;
  };
}

function where(properties) {
  var entries = Object.entries(properties);
  return filter(function (datum) {
    if (datum == null) {
      return false;
    }

    return entries.every(function (_ref) {
      var key = _ref[0],
          value = _ref[1];
      return datum[key] === value;
    });
  });
}

function zip() {
  return /*#__PURE__*/_regeneratorRuntime.mark(function zipFn(data) {
    var iterators, values;
    return _regeneratorRuntime.wrap(function zipFn$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // Map each item of `data` to an iterator
            iterators = Array.from(data).map(function (datum) {
              return datum[Symbol.iterator]();
            });

          case 1:

            // Take the next value of each iterator
            values = iterators.map(function (datum) {
              return datum.next();
            }); // Stop once some values are done

            if (!values.some(function (value) {
              return value.done;
            })) {
              _context.next = 5;
              break;
            }

            return _context.abrupt("return");

          case 5:
            _context.next = 7;
            return values.map(function (value) {
              return value.value;
            });

          case 7:
            _context.next = 1;
            break;

          case 9:
          case "end":
            return _context.stop();
        }
      }
    }, zipFn);
  });
}

exports.average = average;
exports.chunk = chunk;
exports.compact = compact;
exports.compose = compose;
exports.concat = concat;
exports.delay = delay;
exports.every = every;
exports.filter = filter;
exports.find = find;
exports.findIndex = findIndex;
exports.first = first;
exports.flatten = flatten;
exports.generate = generate;
exports.groupBy = groupBy;
exports.head = head;
exports.map = map;
exports.max = max;
exports.mean = mean;
exports.min = min;
exports.partition = partition;
exports.pipe = pipe;
exports.range = range;
exports.reduce = reduce;
exports.reverse = reverse;
exports.skip = skip;
exports.slice = slice;
exports.some = some;
exports.sum = sum;
exports.take = take;
exports.takeWhile = takeWhile;
exports.tap = tap;
exports.toArray = toArray;
exports.unique = unique;
exports.where = where;
exports.zip = zip;
//# sourceMappingURL=lazy-collections.cjs.development.js.map
